@@ -30,6 +30,15 @@ class TestPackage(unittest.TestCase):
         pkg = upt.Package('foo', '4.2', archives=archives)
         self.assertEqual(pkg.get_archive(), archives[0])
 
+    def test_get_archive_with_argument(self):
+        archives = [
+            upt.Archive('url', archive_type=upt.ArchiveType.RUBYGEM)
+        ]
+        pkg = upt.Package('foo', '4.2', archives=archives)
+        with self.assertRaises(upt.ArchiveUnavailable):
+            pkg.get_archive()
+        self.assertEqual(pkg.get_archive(upt.ArchiveType.RUBYGEM), archives[0])
+
     @mock.patch('os.remove')
     def test_clean_archive_downloaded(self, remove_fn):
         archives = [
@@ -110,33 +119,45 @@ class TestUtils(unittest.TestCase):
 
 
 class TestParser(unittest.TestCase):
-    def setUp(self):
-        self.parser = upt.upt.create_parser()
+    def test_package_single_frontend(self):
+        parser = upt.upt.create_parser(['pypi'], ['guix', 'nix'])
+        args = 'package -b guix requests'.split()
+        parser.parse_args(args)
+
+    def test_package_single_backend(self):
+        parser = upt.upt.create_parser(['cpan', 'pypi'], ['guix'])
+        args = 'package -f pypi requests'.split()
+        parser.parse_args(args)
 
     def test_package_missing_frontend(self):
+        parser = upt.upt.create_parser(['cpan', 'pypi'], ['guix', 'nix'])
         args = 'package -b guix requests'.split()
         with self.assertRaises(SystemExit):
-            self.parser.parse_args(args)
+            parser.parse_args(args)
 
     def test_package_missing_backend(self):
+        parser = upt.upt.create_parser(['cpan', 'pypi'], ['guix', 'nix'])
         args = 'package -f pypi requests'.split()
         with self.assertRaises(SystemExit):
-            self.parser.parse_args(args)
+            parser.parse_args(args)
 
     def test_package_missing_frontend_and_backend(self):
+        parser = upt.upt.create_parser(['cpan', 'pypi'], ['guix', 'nix'])
         args = 'package requests'.split()
         with self.assertRaises(SystemExit):
-            self.parser.parse_args(args)
+            parser.parse_args(args)
 
     def test_package_missing_package(self):
+        parser = upt.upt.create_parser(['cpan', 'pypi'], ['guix', 'nix'])
         args = 'package -f pypi -b guix'.split()
         with self.assertRaises(SystemExit):
-            self.parser.parse_args(args)
+            parser.parse_args(args)
 
     def test_package_exclusive_logging_options(self):
+        parser = upt.upt.create_parser(['cpan', 'pypi'], ['guix', 'nix'])
         args = 'package -f pypi -b guix --debug --quiet requests'.split()
         with self.assertRaises(SystemExit):
-            self.parser.parse_args(args)
+            parser.parse_args(args)
 
 
 class TestCommandLine(unittest.TestCase):
@@ -186,24 +207,43 @@ class TestCommandLine(unittest.TestCase):
     @mock.patch('upt.upt._get_installed_frontends', return_value={})
     @mock.patch('upt.upt._get_installed_backends',
                 return_value={'valid': mock.Mock()})
-    def test_package_invalid_frontend(self, m_backends, m_frontends):
-        sys.argv.extend('package -f invalid -b valid pkgname'.split())
-        with self.assertRaises(SystemExit) as exit:
+    def test_package_no_frontends(self, m_backends, m_frontends):
+        sys.argv.extend('package'.split())
+        with self.assertRaises(SystemExit):
             upt.upt.main()
-        self.assertEqual(exit.exception.code, 1)
 
     @mock.patch('upt.upt._get_installed_frontends',
                 return_value={'valid': mock.Mock()})
     @mock.patch('upt.upt._get_installed_backends', return_value={})
-    def test_package_invalid_backend(self, m_backends, m_frontends):
-        sys.argv.extend('package -f valid -b invalid pkgname'.split())
-        with self.assertRaises(SystemExit) as exit:
+    def test_package_no_backends(self, m_backends, m_frontends):
+        sys.argv.extend('package'.split())
+        with self.assertRaises(SystemExit):
             upt.upt.main()
-        self.assertEqual(exit.exception.code, 1)
 
     @mock.patch('upt.upt._get_installed_frontends',
                 return_value={'valid-frontend': mock.Mock()})
-    @mock.patch('upt.upt._get_installed_backends')
+    @mock.patch('upt.upt._get_installed_backends',
+                return_value={'valid-backend': mock.Mock()})
+    def test_package_invalid_frontend(self, m_backends, m_frontends):
+        sys.argv.extend('package -f invalid -b valid-backend pkgname'.split())
+        with self.assertRaises(SystemExit) as exit:
+            upt.upt.main()
+        self.assertEqual(exit.exception.code, 2)
+
+    @mock.patch('upt.upt._get_installed_frontends',
+                return_value={'valid-frontend': mock.Mock()})
+    @mock.patch('upt.upt._get_installed_backends',
+                return_value={'valid-backend': mock.Mock()})
+    def test_package_invalid_backend(self, m_backends, m_frontends):
+        sys.argv.extend('package -f valid-frontend -b invalid pkgname'.split())
+        with self.assertRaises(SystemExit) as exit:
+            upt.upt.main()
+        self.assertEqual(exit.exception.code, 2)
+
+    @mock.patch('upt.upt._get_installed_frontends',
+                return_value={'valid-frontend': mock.Mock()})
+    @mock.patch('upt.upt._get_installed_backends',
+                return_value={'valid-backend': mock.Mock()})
     def test_package_unhandled_frontend(self, m_backends, m_frontends):
         cmdline = 'package -f valid-frontend -b valid-backend pkgname'
         sys.argv.extend(cmdline.split())
@@ -222,18 +262,19 @@ class TestCommandLine(unittest.TestCase):
 
     @mock.patch('upt.upt._get_installed_frontends',
                 return_value={'valid-frontend': mock.Mock()})
-    @mock.patch('upt.upt._get_installed_backends')
+    @mock.patch('upt.upt._get_installed_backends',
+                return_value={'valid-backend': mock.Mock()})
     def test_package_invalid_pkgname(self, m_backends, m_frontends):
         cmdline = 'package -f valid-frontend -b valid-backend pkgname'
         sys.argv.extend(cmdline.split())
 
-        def backend_fail(*args, **kwargs):
+        def frontend_fail(*args, **kwargs):
             raise upt.exceptions.InvalidPackageNameError('valid-backend',
                                                          'pkgname')
-        fake_backend = mock.Mock()
-        fake_backend().create_package.side_effect = backend_fail
-        m_backends.return_value = {
-            'valid-backend': fake_backend
+        fake_frontend = mock.Mock()
+        fake_frontend().parse.side_effect = frontend_fail
+        m_frontends.return_value = {
+            'valid-frontend': fake_frontend
         }
         with self.assertRaises(SystemExit) as exit:
             upt.upt.main()
@@ -254,15 +295,23 @@ class TestArchive(unittest.TestCase):
         archive._filepath = '/fake/path'
         self.assertEqual(archive.size, 12)
 
-    @mock.patch('urllib.request.urlretrieve', return_value=('/path', None))
-    def test_filepath(self, urlretrieve_fn):
-        archive = upt.Archive('url')
-        self.assertEqual(archive.filepath, '/path')
+    def test_filename(self):
+        archive = upt.Archive('http://example.com/source.tar.gz')
+        self.assertEqual(archive.filename, 'source.tar.gz')
+
+    @mock.patch('urllib.request.urlretrieve',
+                return_value=('/tmpdir/source.tar.gz', None))
+    @mock.patch('tempfile.gettempdir', return_value='/tmpdir')
+    def test_filepath(self, gettempdir_fn, urlretrieve_fn):
+        archive = upt.Archive('http://example.com/source.tar.gz')
+        self.assertEqual(archive.filepath, '/tmpdir/source.tar.gz')
 
     @mock.patch('upt.checksum.compute_checksum', return_value='hash-output')
-    @mock.patch('urllib.request.urlretrieve', return_value=('/path', None))
-    def test_checksum(self, urlretrieve_fn, compute_checksum_fn):
-        archive = upt.Archive('url')
+    @mock.patch('urllib.request.urlretrieve',
+                return_value=('/tmpdir/source.tar.gz', None))
+    @mock.patch('tempfile.gettempdir', return_value='/tmpdir')
+    def test_checksum(self, gettmpdir_fn, urlretrieve_fn, compute_checksum_fn):
+        archive = upt.Archive('http://example.com/source.tar.gz')
         self.assertEqual(archive._checksum('fake-hash'), 'hash-output')
         self.assertEqual(archive._checksum('fake-hash'), 'hash-output')
         urlretrieve_fn.assert_called_once()
@@ -271,8 +320,9 @@ class TestArchive(unittest.TestCase):
     @mock.patch('upt.checksum.compute_checksum', return_value='hash-output')
     def test_checksums_provided(self, compute_checksum_fn):
         archive = upt.Archive('url', md5='md5', rmd160='rmd160',
-                              sha256='sha256')
+                              sha256='sha256', sha256_base64='sha256_base64')
         self.assertEqual(archive.md5, 'md5')
         self.assertEqual(archive.rmd160, 'rmd160')
         self.assertEqual(archive.sha256, 'sha256')
+        self.assertEqual(archive.sha256_base64, 'sha256_base64')
         compute_checksum_fn.assert_not_called()
