@@ -11,6 +11,7 @@ from upt import checksum
 class TestChecksum(unittest.TestCase):
     MD5 = '3858f62230ac3c915f300c664312c63f'
     SHA256 = 'c3ab8ff13720e8ad9047dd39466b3c8974e592c2fa383d4a3960714caef0c4f2'
+    SHA256_BASE64 = 'w6uP8Tcg6K2QR905Rms8iXTlksL6OD1KOWBxTK7wxPI='
 
     def setUp(self):
         self.path = '/fake/path'
@@ -22,6 +23,10 @@ class TestChecksum(unittest.TestCase):
         self.assertEqual(checksum._compute_sha256_checksum(self.path),
                          self.SHA256)
 
+    def test_sha256_base64_checksum(self, open_fn):
+        self.assertEqual(checksum._compute_sha256_base64_checksum(self.path),
+                         self.SHA256_BASE64)
+
     def test_compute_checksum(self, open_fn):
         self.assertEqual(checksum.compute_checksum(self.path, 'md5'), self.MD5)
 
